""" This module is loaded instead of ufl when executing .ufl files.
So, this module contains all our extensions and monkey patches to
UFL.
"""
from dune.perftool.error import PerftoolUFLError

import ufl

from ufl import *


class TrialFunction(ufl.Coefficient):
    """ A coefficient that always takes the reserved index 0 """
    def __init__(self, element, count=None):
        if count and count is not 0:
            raise PerftoolUFLError("The trial function must be the coefficient of index 0 in uflpdelab")
        ufl.Coefficient.__init__(self, element, count=0)


class Coefficient(ufl.Coefficient):
    """ A coefficient that honors the reserved index 0. """
    def __init__(self, element, count=None):
        if count and count is 0:
            raise PerftoolUFLError("The coefficient of index 0 is reserved for the trial function in uflpdelab")
        if count and count is 1:
            raise PerftoolUFLError("The coefficient of index 1 is reserved for the jacobian apply vector in uflpdelab")
        if not count and ufl.Coefficient._globalcount < 2:
            count = 2
        ufl.Coefficient.__init__(self, element, count)


def split(obj):
    return ufl.split_functions.split2(obj)


def Coefficients(element):
    return split(Coefficient(element))


def TestFunctions(element):
    return split(TestFunction(element))


def TrialFunctions(element):
    return split(TrialFunction(element))
